SUBROUTINE ENDPOS(KPROMA,KST,KND,KFPDYN,KAFPOS_GFL,KOPLEV,CDCONF,PXLEV,YDIN_GFL,&
 & PGFLT0,PUT0,PVT0,PTT0,PDEP0,PWVEL0,POROG,PSPT0,&
 & PTS0,PITS0,PSD0,PTXI,PTNI,PSW0,PSF0,PHV0,PSR0,PTCLS,PRHCLS,&
 & PTR0,PHUXI,PHUNI,&
 & PCFIS,PCLSM,PCVEG,PCIVE,PCARG,PCD2,PCSAB,PGM,&
 & PFP)  

!**** *ENDPOS* - End of post-processing

!     PURPOSE.
!     --------
!       Post-processing calculations after horizontal interpolations of 
!        model fields ; used to compute fields on surface-dependent levels
!        (height, eta levels or PBL physico-dynamic fields)

!**   INTERFACE.
!     ----------
!        *CALL* *ENDPOS(...)

!        EXPLICIT ARGUMENTS :
!        --------------------
!        * INPUT:
!        KPROMA    : horizontal dimension
!        KST       : starting index of work
!        KND       : depth of work
!        KFPDYN    : Number of fields in output array
!        KAFPOS_GFL: Second dimension of PGFLT0.
!        KOPLEV    : number of post-processing levels
!        CDCONF    : Conf of work : 'Z' = height levels ; 'E' = eta levels
!        PXLEV     : array containing the pp levels (heights or eta indexes)
!        YDIN_GFL  : pointers to locate individual GFL in PGFLT0.
!   --  horizontally interpolated model variables ---
!        PGFLT0    : GFL variables
!        PUT0      : zonal wind
!        PVT0      : meridional wind
!        PTT0      : temperature 
!        PDEP0     : n.-h. pressure departure
!        PWVEL0    : true vertical velocity w at half levels
!        POROG     : surface geopotential
!        PSPT0     : surface pressure variable
!        PTS0      : output surface temperature
!        PITS0     : interpolated surface temperature
!        PSD0      : output snow depth
!        PTXI      : increment (Tx-Tcls)
!        PTNI      : increment (Tn-Tcls)
!        PSW0      : output surface water content
!        PSF0      : output surface frost
!        PHV0      : output resistance to evapotranspiration
!        PSR0      : output model dynamical roughness length times g
!        PTCLS     : interpolated cls temperature
!        PRHCLS    : interpolated cls relative moisture
!        PTR0      : output model thermal roughness length times g
!        PHUXI     : increment (HUx-HUcls)
!        PHUNI     : increment (HUn-HUcls)
!        PCFIS     : output orography
!        PCLSM     : output land-sea mask
!        PCVEG     : climatology percentage of vegetation
!        PCIVE     : climatology index of vegetation
!        PCARG     : climatology percentage of silt
!        PCD2      : climatology soild depth
!        PCSAB     : climatology percentage of sand
!        PGM       : map factor
!        * OUTPUT:
!        PFP       : output fields array

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!      RYAD EL KHATIB *METEO-FRANCE*
!      ORIGINAL : 94-04-08

!     MODIFICATIONS.
!     --------------
!      F. Vana   : 02-07-04 Virtual potential temperature
!      R. El Khatib : 02-04-07 ICAO fields moved from POS to here ; iso -10.
!      R. El Khatib : 03-08-13 gfl
!      R. El Khatib : 03-04-17 Fullpos improvemnts
!      R. El Khatib : 03-08-27 computation of wetpoints up to 300 hpa only.
!      M.Hamrud : 01-Oct-2003 CY28 Cleaning
!      R. Engelen : 15-Mar-2005 Greenhouse gases
!      A. Untch : 15-Mar-2005 Aerosols
!      J. Flemming: 11-Apr-2005 Aerosol replaced with GEMS GRG 
!      Y. Bouteloup : 05-01-28 Introduction of precipitation (rain)
!      G. Hello     : 05-01-28 snow, graupel and tke (and rain too !)
!      G. Hello     : 06-08-31 simulated reflectivities
!      G. Hello     : 07-02 thetav
!      N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!      Y. Seity     : 20-Dec-2007 hail
!      R. El Khatib : 05-May-2008 Bugfixes on recent developments
!      F. Vana      : 27-Aug-2008 vectorization support for NEC
!      K. Yessad (Aug 2009): rewrite vertical interpolator in post-processing.
!      Y. Bouteloup  : 14-07-2009 Add radiative cloud water and ice (YIRAD and YLRAD)
!      F. Bouyssel  : 15-Jul-2009 LFPCAPEX
!      K. Yessad (Dec 2011): use GPHPRE.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSVX2
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG, NPPM
USE YOMDYNCORE, ONLY : LDYNCORE
USE YOMFPC   , ONLY : NFPCLI   ,LFPRH100 ,FPBL     ,LFPQ     ,NFPCAPE  ,&
 & FPRHMIN  ,FPRHMAX  ,LFPCAPEX ,LFPTTOP ,NFP3I    ,RFP3I
USE YOMFPG   , ONLY : FPVALH   ,FPVBH    ,NFPLEV
USE YOMPHY   , ONLY : LMPHYS
USE YOMGEM   , ONLY : YRVAB
USE YOMSTA   , ONLY : RDTDZ1   ,NLEXTRAP ,STPRE
USE YOMCST   , ONLY : RG       ,RTT
USE YOMVPOS  , ONLY : QFPTYPE
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMAFN   , ONLY : TFP_SP   ,TFP_LNSP ,TFP_U    ,TFP_V    ,TFP_T    ,&
 & TFP_Z    ,TFP_HU   ,TFP_TH   ,TFP_THPW ,TFP_ETH  ,TFP_WND  ,TFP_P    ,&
 & TFP_PD   ,TFP_VD   ,TFP_VW   ,TFP_FUA  ,TFP_WWS  ,TFP_RHO  ,&
 & TFP_UCLS ,TFP_VCLS ,TFP_TCLS ,TFP_QCLS ,TFP_RCLS ,TFP_FCLS ,&
 & TFP_UGST ,TFP_VGST ,TFP_FGST ,TFP_PTB ,TFP_HTB ,TFP_HTPW ,&
 & TFP_CAPE ,TFP_CIEN ,TFP_TX   ,TFP_TN   ,TFP_HUX  ,TFP_HUN  ,&
 & TFP_UJET ,TFP_VJET ,TFP_PJET ,TFP_PCAO ,TFP_TCAO ,TFP_IET  ,&
 & TFP_DYNDS,TFP_SRE, TFP_THV ,TFP_TCVS ,&
 & TFP_LCL  ,TFP_FCL  , TFP_EL
USE YOM_YGFL , ONLY : NGFL_EXT, NAERO, NGRG, NGHG
USE TYPE_GFLFLDS, ONLY: TYPE_IGFLFLD
USE INTDYN_MOD,ONLY : YYTXYB

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST
INTEGER(KIND=JPIM),INTENT(IN)    :: KND
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPDYN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAFPOS_GFL
INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXLEV(KOPLEV) 
TYPE(TYPE_IGFLFLD),INTENT(IN)    :: YDIN_GFL
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFLT0(KPROMA,0:KAFPOS_GFL)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDEP0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWVEL0(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PITS0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTXI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSW0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSF0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHV0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSR0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTR0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHUXI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHUNI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFIS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVEG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCIVE(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCARG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCD2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCSAB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFP(KPROMA,KFPDYN) 

!     ------------------------------------------------------------------

! === LOGICAL VARIABLES COMPUTED IN PART 1.1.4 =========================
LOGICAL :: LL_ETH   ! equivalent potential temperature
LOGICAL :: LL_IET   ! isobaric equivalent temperature
LOGICAL :: LL_SRE   ! simulated reflectivities
LOGICAL :: LL_THV   ! virtual theta
LOGICAL :: LL_PD    ! NH pressure departure
LOGICAL :: LL_HU    ! Relative humidity
LOGICAL :: LL_TH    ! Potential temperature
LOGICAL :: LL_THPW  ! Wet bulb potential temperature
LOGICAL :: LL_VD    ! Vertical divergence
LOGICAL :: LL_VW    ! True vertical velocity
LOGICAL :: LL_WND   ! wind speed
LOGICAL :: LL_WWS   ! surface true vertical velocity 
LOGICAL :: LL_Z     ! Geopotential
LOGICAL :: LL_HTPW  ! height of iso-T'w=0 Celsius, from bottom
LOGICAL :: LL_UJET  ! ICAO U component of the jet
LOGICAL :: LL_VJET  ! ICAO V component of the jet
LOGICAL :: LL_PJET  ! ICAO pressure at jet level
LOGICAL :: LL_PCAO  ! ICAO tropopause pressure
LOGICAL :: LL_TCAO  ! ICAO tropopause temperature 
LOGICAL :: LL_PTB   ! Pressure of iso-T
LOGICAL :: LL_HTB   ! height of iso-T
LOGICAL :: LL_RHO

! === OTHER LOGICAL VARIABLES ==========================================
LOGICAL :: LLPBL ! to activate PBL computation
LOGICAL :: LLICV ! to activate the computation of CAPE & CIEN & TCVS
LOGICAL :: LL1,LL2 
LOGICAL :: LLWFULL ! to compute w at full levels
LOGICAL :: LLWHALF ! to compute w at half levels
LOGICAL :: LLGEOP  ! to compute geopotential
LOGICAL :: LLNHPD  ! to compute pressure departure (NH)
LOGICAL :: LLSPCH  ! to interpolate Q rather than Hu
LOGICAL,ALLOCATABLE :: LL_GFL(:) ! GFL
LOGICAL,ALLOCATABLE :: LLS1_GFL(:),LLS2_GFL(:),LLIN_GFL(:)
LOGICAL :: LLTOP, LLAERO, LLUNIT, LLAPA(2)

! === REAL ARRAYS COMPUTED IN PART 1.2 =================================
REAL(KIND=JPRB),ALLOCATABLE :: ZGFLT0(:,:,:)
REAL(KIND=JPRB) :: ZUT0 (KPROMA,0:NFLEVG), ZVT0 (KPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZTT0 (KPROMA,0:NFLEVG),ZGRHLT0(KPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZDEP0(KPROMA,0:NFLEVG), ZWVEL0(KPROMA,0:NFLEVG)

! === REAL ARRAYS COMPUTED IN PART 1.3 =================================
REAL(KIND=JPRB) :: ZI_PRESH(KPROMA,0:NFLEVG), ZI_PRESF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZI_XYB(KPROMA,NFLEVG,YYTXYB%NDIM)
REAL(KIND=JPRB) :: ZI_GEOPH(KPROMA,0:NFLEVG),ZI_GEOPF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZI_R(KPROMA,NFLEVG),ZI_RRED(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZI_CP(KPROMA,NFLEVG),ZI_KAPA(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZI_RHF(KPROMA,0:NFLEVG),ZUSL_ES(KPROMA,NFLEVG)

! === REAL ARRAYS COMPUTED IN PARTS 1.4 AND 1.5 ========================
REAL(KIND=JPRB) :: ZOOROG(KPROMA) ! output orography
REAL(KIND=JPRB) :: ZTSI  (KPROMA) ! interpolated surface temperature without
                                  ! altitude correction
REAL(KIND=JPRB) :: ZTSA  (KPROMA) ! interpolated surface temperature WITH
                                  ! altitude correction 
REAL(KIND=JPRB) :: ZTSO  (KPROMA) ! output surface temperature
REAL(KIND=JPRB) :: ZTSEA (KPROMA) ! sea surface temperature
REAL(KIND=JPRB) :: ZSPP(KPROMA)   ! surface pressure
REAL(KIND=JPRB) :: ZLNSP(KPROMA)  ! Ln of surface pressure

! === OUTPUT OF PART 2 =================================================
! Arrays dimensioned with MAX(NFLEVG,KOPLEV) are used in both parts 2.1 and 2.2;
! arrays dimensioned with NFLEVG (resp. KOPLEV) are used in part 2.1 (resp. 2.2)
REAL(KIND=JPRB), ALLOCATABLE :: ZWHPP2(:,:,:) ! w half levels
REAL(KIND=JPRB), ALLOCATABLE :: ZPREH2(:,:,:) ! half levels pressures
REAL(KIND=JPRB) :: ZXLEVG(NFLEVG) ! level indexes
REAL(KIND=JPRB) :: ZPREPP(KPROMA,MAX(NFLEVG,KOPLEV)) ! upper air pressure
REAL(KIND=JPRB) :: ZUPP(KPROMA,MAX(NFLEVG,KOPLEV))   ! U
REAL(KIND=JPRB) :: ZVPP(KPROMA,MAX(NFLEVG,KOPLEV))   ! V
REAL(KIND=JPRB) :: ZTPP(KPROMA,MAX(NFLEVG,KOPLEV))   ! T
REAL(KIND=JPRB) :: ZPDEPP(KPROMA,MAX(NFLEVG,KOPLEV)) ! NH pressure departure
REAL(KIND=JPRB) :: ZWFPP(KPROMA,MAX(NFLEVG,KOPLEV))  ! w full levels
REAL(KIND=JPRB),ALLOCATABLE :: ZGFLPP(:,:,:)         ! GFL
REAL(KIND=JPRB) :: ZPHIPP(KPROMA,MAX(NFLEVG,KOPLEV)) ! geopotential
REAL(KIND=JPRB) :: ZHUPP(KPROMA,MAX(NFLEVG,KOPLEV))  ! Hu
REAL(KIND=JPRB) :: ZGRHLPP(KPROMA,MAX(NFLEVG,KOPLEV))! Hail+Graupel
REAL(KIND=JPRB) :: ZWSPP(KPROMA)            ! surface true vertical velocity
REAL(KIND=JPRB) :: ZKAPPP(KPROMA,MAX(NFLEVG,KOPLEV)) ! Kappa on output full levels
REAL(KIND=JPRB) :: ZRPP(KPROMA,MAX(NFLEVG,KOPLEV))   ! R on output full levels
REAL(KIND=JPRB) :: ZCPPP(KPROMA,MAX(NFLEVG,KOPLEV))  ! Cp on output full levels
REAL(KIND=JPRB) :: ZPRES_PPH(KPROMA,0:NFLEVG),ZPRES_PPF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZXYB_PP(KPROMA,NFLEVG,YYTXYB%NDIM)
REAL(KIND=JPRB) :: ZRRED_PP(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGEO_PPH(KPROMA,0:NFLEVG),ZGEO_PPF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTETAPP(KPROMA,KOPLEV)   ! Theta
REAL(KIND=JPRB) :: ZEPTHPP(KPROMA,KOPLEV)   ! equivalent potential temperature
REAL(KIND=JPRB) :: ZIBETPP(KPROMA,KOPLEV)   ! virtual potential temperature
REAL(KIND=JPRB) :: ZTHPWPP(KPROMA,KOPLEV)   ! wet bulb potential temperature
REAL(KIND=JPRB) :: ZWINDPP(KPROMA,KOPLEV)   ! hirizontal wind velocity
REAL(KIND=JPRB) :: ZSIMRPP(KPROMA,KOPLEV)   ! simulated reflectivities
REAL(KIND=JPRB) :: ZVIRTPP(KPROMA,KOPLEV)   ! virtual theta
REAL(KIND=JPRB) :: ZMGDWPP(KPROMA,KOPLEV)   ! -G*(Delta w)
REAL(KIND=JPRB) :: ZRHOPP(KPROMA,KOPLEV)  
REAL(KIND=JPRB) :: ZTPWPP(KPROMA,MAX(NFLEVG,KOPLEV)) ! T'w
REAL(KIND=JPRB) :: ZPEORG(KPROMA,MAX(NFLEVG,KOPLEV)) ! pressures of data origin
REAL(KIND=JPRB) :: ZR2PP(KPROMA,MAX(NFLEVG,KOPLEV))
REAL(KIND=JPRB) :: ZQCLS(KPROMA) ! Q of pbl
REAL(KIND=JPRB) :: ZRHCLS(KPROMA)! RH of pbl
REAL(KIND=JPRB) :: ZTCLS(KPROMA) ! T of pbl
REAL(KIND=JPRB) :: ZUCLS(KPROMA) ! U of pbl
REAL(KIND=JPRB) :: ZVCLS(KPROMA) ! V of pbl
REAL(KIND=JPRB) :: ZFCLS(KPROMA) ! wind velocity of pbl
REAL(KIND=JPRB) :: ZPCLS(KPROMA) ! pressure of pbl
REAL(KIND=JPRB) :: ZCAPE(KPROMA) ! CAPE
REAL(KIND=JPRB) :: ZCIN(KPROMA)  ! CIN
REAL(KIND=JPRB) :: ZTCVS(KPROMA) ! temperature of convection
REAL(KIND=JPRB) :: ZUGST(KPROMA) ! U of gusts
REAL(KIND=JPRB) :: ZVGST(KPROMA) ! V of gusts
REAL(KIND=JPRB) :: ZFGST(KPROMA) ! gusts
INTEGER(KIND=JPIM) :: ILCL(KPROMA) ! Lifting condensation level
INTEGER(KIND=JPIM) :: IFCL(KPROMA) ! Free convection level
INTEGER(KIND=JPIM) :: IEL(KPROMA)  ! Equilibrium level
REAL(KIND=JPRB) :: ZLCL(KPROMA) !  Lifting condensation level (height)
REAL(KIND=JPRB) :: ZFCL(KPROMA) ! Free convection level (height)
REAL(KIND=JPRB) :: ZEL(KPROMA)  ! Equilibrium level (height)
REAL(KIND=JPRB) :: ZHTPW(KPROMA) ! height of iso-T'w=0 Celsius, from bottom
REAL(KIND=JPRB) :: ZPICAO(KPROMA)! ICAO tropopause - pressure
REAL(KIND=JPRB) :: ZTICAO(KPROMA)! ICAO tropopause - temperature
REAL(KIND=JPRB) :: ZUJET(KPROMA) ! ICAO jet - U momentum
REAL(KIND=JPRB) :: ZVJET(KPROMA) ! ICAO jet - V momentum
REAL(KIND=JPRB) :: ZPJET(KPROMA) ! ICAO jet - pressure 
REAL(KIND=JPRB) :: ZTMIN(KPROMA) ! Tcls min
REAL(KIND=JPRB) :: ZTMAX(KPROMA) ! Tcls max
REAL(KIND=JPRB) :: ZHUMIN(KPROMA)! Hu cls min
REAL(KIND=JPRB) :: ZHUMAX(KPROMA)! Hu cls max
REAL(KIND=JPRB) :: ZGEOP(KPROMA,KOPLEV) ! geopotentials of the height levels
REAL(KIND=JPRB) :: ZPTB(KPROMA,NFP3I)   ! Pressure of iso-T
REAL(KIND=JPRB) :: ZHTB(KPROMA,NFP3I)   ! Height of iso-T

! === OTHER LOCAL VARIABLES ============================================

REAL(KIND=JPRB), PARAMETER :: PPROOF=30000._JPRB

REAL(KIND=JPRB) :: ZPPIN(KPROMA,MAX(NFLEVG,KOPLEV))
REAL(KIND=JPRB) :: Z1SGM(KPROMA) ! Reverse of map factor
REAL(KIND=JPRB) :: ZRHBNDS(2)

REAL(KIND=JPRB) :: ZDTDPHI  ! dT/(g*dz1)
REAL(KIND=JPRB) :: Z1SG     ! 1/g
REAL(KIND=JPRB) :: ZXTEMP

INTEGER(KIND=JPIM) :: JL, JI, JV, JLEVP
INTEGER(KIND=JPIM) :: ILOC, ILPT, I, ICAPE, IPOL2, IMAXLEV, ITOP, IPPGFL
INTEGER(KIND=JPIM),ALLOCATABLE :: IGFLCOD(:),IDIN_GFL(:)

TYPE(TYPE_IGFLFLD) :: YLPPIC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "apache.intfb.h"
#include "ctstar.intfb.h"
#include "endpos_prepgfl.intfb.h"
#include "fpachmt.intfb.h"
#include "fpcica.intfb.h"
#include "fpps.intfb.h"
#include "gpept.intfb.h"
#include "gpgeo.intfb.h"
#include "gpiet.intfb.h"
#include "gphpre.intfb.h"
#include "gpprs0d.intfb.h"
#include "gprcp.intfb.h"
#include "gprh.intfb.h"
#include "gptet.intfb.h"
#include "poaero.intfb.h"
#include "ppcvirt.intfb.h"
#include "ppleta.intfb.h"
#include "ppltemp.intfb.h"
#include "ppthpw.intfb.h"
#include "ppwetpoint.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ENDPOS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       0.    PRELIMINARY TESTS
!              -----------------

IF (LRWSDLR.AND.LDYNCORE) THEN
  ! ky: this configuration cannot work properly in this case because
  !  part 1.3 waits for "presims" in input PSPT0, and PSPT0 actually
  !  contains a post-processed value of "prehyds".
  ! In LDYNCORE configurations, like small planet configurations,
  !  "presims" may be very different from "prehyds" and there is no
  !  simple algorithm to convert "prehyds" into "presims"
  !  (the only clean solution would be to use the iterative algorithm
  !  coded in GHN_CONV_PRHS, and adapted to arrays of length KPROMA
  !  which may be different from NPROMA or NGPTOT).
  WRITE (NULOUT,*) ' ENDPOS: not properly adapted for LRWSDLR=T!!'
  CALL ABOR1(' ENDPOS 0a: Case not supported in ENDPOS')
ENDIF

IF (LRWSDLR.AND..NOT.LDYNCORE) THEN
  ! ky: NH deep-layer model on a real planet.
  !  "presims" is assumed to be sufficiently close to "prehyds", in
  !  order to allow to call GPHPRE with "prehyds" instead
  !  of "presims". Difference presim-prehyd is neglected in calculations
  !  done in part 1.3.
  ! We recall that Dpresim/Dprehyd=(g/G)((a+z)**2/a**2) and
  !  presim_top=prehyd_top, where "z" is the altitude.
  ! In TL799L91c1 simulations (ECMWF oper model in 2009)
  !  presims-prehyds is around 200Pa when g=G everywhere (vertical variations
  !  of "g" ignored). This difference may be bigger if vertical variations
  !  of "g" are taken into account (LRWSDLG=T).
  ! Calculations done in part 1.3 ignore LRWSDLR=T deep-layer effects.
  WRITE (NULOUT,*) ''
  WRITE (NULOUT,*) ' ENDPOS: calculations in part 1.3 are only approximative for LRWSDLR=T'
  WRITE (NULOUT,*) ' FULL-POS configurations calling ENDPOS are not recommended for LRWSDLR=T'
  WRITE (NULOUT,*) ''
ENDIF

!     ------------------------------------------------------------------

!*       1.    INITIALIZATIONS.
!              ---------------

!*    1.1  LOGICAL AND OTHER SCALAR INITIALIZATIONS
  
!     1.1.1  Define the "standard order" and pointers for post-processed GFL

! we compute here pointers YLPPIC%I[X] to locate GFL in LL_GFL and ZGFLPP,
! and the number of 2D fields (IPPGFL) we have to store in LL_GFL and ZGFLPP.
! Assumptions:
! - memory is conditionally allocated for multi-fields GFL.
! - for the other GFL variables allocation is always done for the time being,
!   even for not post-processed fields.
! These pointers will be re-used for the non-derivative fields stored in ZGFLT0.

IPPGFL=0
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IQ)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IL)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%ILRAD)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%II)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IIRAD)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IA)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IO3)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IS)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IRR)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IG)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%IH)
CALL SUPTRPPGFL_ENDPOS(1,IPPGFL,YLPPIC%ITKE)
CALL SUPTRPPGFL_ENDPOS(NGHG,IPPGFL,YLPPIC%IGHG)
CALL SUPTRPPGFL_ENDPOS(NAERO,IPPGFL,YLPPIC%IAERO)
CALL SUPTRPPGFL_ENDPOS(NGRG,IPPGFL,YLPPIC%IGRG)
CALL SUPTRPPGFL_ENDPOS(NGFL_EXT,IPPGFL,YLPPIC%IEXT)

!     1.1.2  LLICV,LLPBL,LL1,LL2.

LLICV = QFPTYPE%LL(TFP_CAPE%ICOD) .OR. QFPTYPE%LL(TFP_CIEN%ICOD) &
 & .OR. QFPTYPE%LL(TFP_LCL%ICOD)  .OR. QFPTYPE%LL(TFP_FCL%ICOD) &
 & .OR. QFPTYPE%LL(TFP_EL%ICOD)   .OR. QFPTYPE%LL(TFP_TCVS%ICOD)

LLPBL = QFPTYPE%LL(TFP_UCLS%ICOD) .OR. QFPTYPE%LL(TFP_VCLS%ICOD)&
 & .OR. QFPTYPE%LL(TFP_TCLS%ICOD) .OR. QFPTYPE%LL(TFP_QCLS%ICOD)&
 & .OR. QFPTYPE%LL(TFP_RCLS%ICOD) .OR. QFPTYPE%LL(TFP_FCLS%ICOD)&
 & .OR. QFPTYPE%LL(TFP_TX%ICOD)   .OR. QFPTYPE%LL(TFP_TN%ICOD)&
 & .OR. QFPTYPE%LL(TFP_HUX%ICOD)  .OR. QFPTYPE%LL(TFP_HUN%ICOD)&
 & .OR. QFPTYPE%LL(TFP_UGST%ICOD) .OR. QFPTYPE%LL(TFP_VGST%ICOD)&
 & .OR. QFPTYPE%LL(TFP_FGST%ICOD) .OR. (LLICV .AND. (.NOT.LFPCAPEX))

LL1=QFPTYPE%LL(TFP_TH%ICOD).OR.QFPTYPE%LL(TFP_ETH%ICOD).OR. &
 & QFPTYPE%LL(TFP_THPW%ICOD).OR.QFPTYPE%LL(TFP_IET%ICOD).OR. &
 & QFPTYPE%LL(TFP_SRE%ICOD).OR.QFPTYPE%LL(TFP_THV%ICOD).OR. &
 & QFPTYPE%LL(TFP_RHO%ICOD)

LL2=QFPTYPE%LL(TFP_TH%ICOD).OR.QFPTYPE%LL(TFP_ETH%ICOD).OR. &
 & QFPTYPE%LL(TFP_IET%ICOD).OR. &
 & QFPTYPE%LL(TFP_SRE%ICOD).OR.QFPTYPE%LL(TFP_THV%ICOD)

!     1.1.3  LL_GFL,LLS1_GFL,LLS2_GFL,LLIN_GFL,IGFLCOD,IDIN_GFL

ALLOCATE(LL_GFL(IPPGFL))
ALLOCATE(LLS1_GFL(IPPGFL))
ALLOCATE(LLS2_GFL(IPPGFL))
ALLOCATE(LLIN_GFL(IPPGFL))
ALLOCATE(IGFLCOD(IPPGFL))
ALLOCATE(IDIN_GFL(IPPGFL))

CALL ENDPOS_PREPGFL(IPPGFL,YDIN_GFL,YLPPIC,LL1,LL2,LLPBL,LLICV,&
 & LL_GFL,LLS1_GFL,LLS2_GFL,LLIN_GFL,IGFLCOD,IDIN_GFL)

!     1.1.4  LL_[X] for other variables 

LL_ETH  = QFPTYPE%LL(TFP_ETH%ICOD)
LL_IET  = QFPTYPE%LL(TFP_IET%ICOD)
LL_SRE  = QFPTYPE%LL(TFP_SRE%ICOD)
LL_THV  = QFPTYPE%LL(TFP_THV%ICOD)
LL_PD   = QFPTYPE%LL(TFP_PD%ICOD)
LL_HU   = QFPTYPE%LL(TFP_HU%ICOD)
LL_TH   = QFPTYPE%LL(TFP_TH%ICOD)
LL_THPW = QFPTYPE%LL(TFP_THPW%ICOD)
LL_VD   = QFPTYPE%LL(TFP_VD%ICOD)
LL_VW   = QFPTYPE%LL(TFP_VW%ICOD)
LL_WND  = QFPTYPE%LL(TFP_WND%ICOD)
LL_WWS  = QFPTYPE%LL(TFP_WWS%ICOD)
LL_Z    = QFPTYPE%LL(TFP_Z%ICOD)

LL_PTB = QFPTYPE%LL(TFP_PTB%ICOD)
LL_HTB = QFPTYPE%LL(TFP_HTB%ICOD)
LL_HTPW = QFPTYPE%LL(TFP_HTPW%ICOD)
LL_UJET = QFPTYPE%LL(TFP_UJET%ICOD)
LL_VJET = QFPTYPE%LL(TFP_VJET%ICOD)
LL_PJET = QFPTYPE%LL(TFP_PJET%ICOD)
LL_PCAO = QFPTYPE%LL(TFP_PCAO%ICOD)
LL_TCAO = QFPTYPE%LL(TFP_TCAO%ICOD)
LL_RHO  = QFPTYPE%LL(TFP_RHO%ICOD)

LLAERO = LL_UJET .OR. LL_VJET .OR. LL_PJET .OR. LL_PCAO .OR. LL_TCAO

!     1.1.5  Other variables.

Z1SG=1.0_JPRB/RG
ZDTDPHI=RDTDZ1/RG

IMAXLEV=MAX(NFLEVG,KOPLEV)

DO JI=KST,KND
  Z1SGM(JI)=1.0_JPRB/PGM(JI)
ENDDO

ZRHBNDS(1)=FPRHMIN
ZRHBNDS(2)=FPRHMAX

!*    1.2  ARRAYS INITIALIZATIONS

!*      1.2.1  GFL 

ALLOCATE(ZGFLT0(KPROMA,0:NFLEVG,IPPGFL))
ZGFLT0(:,:,:)=0.0_JPRB
DO JV=1,IPPGFL
  IF (LLIN_GFL(JV)) THEN
    DO JL=1,NFLEVG
      DO JI=KST,KND
        ZGFLT0(JI,JL,JV)=PGFLT0(JI,IDIN_GFL(JV)+JL-1)
      ENDDO
    ENDDO
  ENDIF
ENDDO
DO JL=1,NFLEVG
  DO JI=KST,KND
    ZGRHLT0(JI,JL)=ZGFLT0(JI,JL,YLPPIC%IG)+ZGFLT0(JI,JL,YLPPIC%IH)
  ENDDO
ENDDO
 
!*      1.2.2  GMV and other non-GFL quantities

DO JL=1, NFLEVG
  DO JI=KST, KND
    ZUT0(JI,JL)=PUT0(JI,JL)*PGM(JI)
    ZVT0(JI,JL)=PVT0(JI,JL)*PGM(JI)
    ZTT0(JI,JL)=PTT0(JI,JL)
    SELECT CASE (TFP_PD%ILMOD)
    CASE (1:)
      ZDEP0(JI,JL)=PDEP0(JI,JL) 
    CASE DEFAULT
      ZDEP0(JI,JL)=0.0_JPRB
    END SELECT
    SELECT CASE (TFP_VW%ILMOD)
    CASE (1:)
      ZWVEL0(JI,JL)=PWVEL0(JI,JL)
    CASE DEFAULT
      ZWVEL0(JI,JL)=0.0_JPRB
    END SELECT
  ENDDO
ENDDO

DO JI=KST, KND
  SELECT CASE (TFP_WWS%ILMOD)
  CASE (1:)
    ZWVEL0(JI,0)=PWVEL0(JI,0)
  CASE DEFAULT
    ZWVEL0(JI,0)=0.0_JPRB
  END SELECT
ENDDO

!*    1.3   MODEL LEVEL PRESSURES AND DYNAMICS

! We find there all calls to GP, GNH routines for departure system.

! Pressures:
ZI_PRESH(KST:KND,NFLEVG)=PSPT0(KST:KND)
CALL GPHPRE(KPROMA,NFLEVG,KST,KND,YRVAB,ZI_PRESH,PXYB=ZI_XYB,PRESF=ZI_PRESF)

! R:
CALL GPRCP(KPROMA,KST,KND,NFLEVG,PQ=ZGFLT0(1,1,YLPPIC%IQ),&
 & PQI=ZGFLT0(1,1,YLPPIC%II),PQL=ZGFLT0(1,1,YLPPIC%IL),&
 & PQR=ZGFLT0(1,1,YLPPIC%IRR),PQS=ZGFLT0(1,1,YLPPIC%IS),PQG=ZGRHLT0(1,1), &
 & PCP=ZI_CP,PR=ZI_R,PKAP=ZI_KAPA) 

! Compute (pre/prehyd)*ZI_R for input to GPGEO.
DO JL=1,NFLEVG
  DO JI=KST,KND
    ZI_RRED(JI,JL)=(1.0_JPRB+ZDEP0(JI,JL)/ZI_PRESF(JI,JL))*ZI_R(JI,JL)
  ENDDO
ENDDO

! Geopotential height:
ZI_GEOPH(KST:KND,NFLEVG)=POROG(KST:KND)
CALL GPGEO(KPROMA,KST,KND,NFLEVG,ZI_GEOPH,ZI_GEOPF,ZTT0(1,1),ZI_RRED, &
 & ZI_XYB(:,:,YYTXYB%M_LNPR),ZI_XYB(:,:,YYTXYB%M_ALPH))

! Relative humidity RH:
CALL GPRH(.FALSE.,KPROMA,KST,KND,NFLEVG,ZRHBNDS(2),ZRHBNDS(1),&
 & ZGFLT0(1,1,YLPPIC%IQ),ZTT0(1,1),ZI_PRESF,ZUSL_ES,ZI_RHF(1,1))

!*    1.4   SET OUTPUT OROGRAPHY AND SURFACE TEMPERATURES

SELECT CASE (NFPCLI)
CASE (0)
  DO JI=KST, KND
    ZOOROG(JI)=POROG(JI)
  ENDDO
CASE DEFAULT
  DO JI=KST, KND
    ZOOROG(JI)=PCFIS(JI)
  ENDDO
END SELECT

SELECT CASE (LMPHYS)
CASE (.TRUE.)
  DO JI=KST, KND
    ZTSI(JI)=PITS0(JI)
    ZTSO(JI)=PTS0(JI)
  ENDDO
CASE DEFAULT
  ! Compute T* :
  CALL CTSTAR(KPROMA,KST,KND,ZTT0(1,NLEXTRAP),ZI_PRESH(1,NFLEVG), &
   & ZI_PRESF(1,NLEXTRAP),POROG,ZTSI,ZTSEA)  
  DO JI=KST, KND
    ZTSO(JI)=ZTSI(JI)+ZDTDPHI*(ZOOROG(JI)-POROG(JI))
  ENDDO
END SELECT

DO JI=KST, KND
  ZTSA(JI)=ZTSI(JI)+ZDTDPHI*(ZOOROG(JI)-POROG(JI))
ENDDO

!*    1.5   PREPARE FOR INTERPOLATIONS: COMPUTE POST-PROCESSING LEVELS

CALL FPPS(KPROMA,KST,KND,NFLEVG,1,ZI_GEOPH,ZOOROG,ZTT0(1,1),&
 & ZI_R,ZI_PRESH,ZTSI,ZSPP)
    
DO JI=KST,KND
  ZLNSP(JI)=LOG(ZSPP(JI))
ENDDO

!*    1.6   ALLOCATIONS.

ALLOCATE(ZGFLPP(KPROMA,IMAXLEV,IPPGFL))

!     ------------------------------------------------------------------

!*       2.    POST-PROCESSING: CALL TO APACHE
!              -------------------------------

!*    2.1   SURFACE & PBL POST-PROCESSING

LLAPA(1)=LL_WWS.OR.LLPBL.OR.LLICV
LLAPA(2)=LL_PTB.OR.LL_HTB.OR.LL_HTPW.OR.LLAERO

IF (LLAPA(1).OR.LLAPA(2)) THEN

  ALLOCATE(ZWHPP2(KPROMA,NFLEVG,0:1))
  ALLOCATE(ZPREH2(KPROMA,NFLEVG,0:1))

  DO JL=1,NFLEVG
    ZXLEVG(JL)=REAL(JL,JPRB)
  ENDDO
  CALL PPLETA(KPROMA,KST,KND,NFLEVG,NFLEVG,ZXLEVG,ZSPP,ZPREPP, &
   & ZPREH2(1,1,0),ZPREH2(1,1,1))

  LLSPCH=LFPQ
  LLWFULL=.FALSE.
  LLNHPD=.FALSE.

ENDIF

IF (LLAPA(1)) THEN

  LLWHALF=LL_WWS
  LLGEOP=.FALSE.

  CALL APACHE(KPROMA,KST,KND,NFLEVG,NFLEVG,IPPGFL,NPPM,YLPPIC,&
   & LLNHPD,LLWHALF,LLWFULL,LLS1_GFL,LLGEOP,LLSPCH,ZRHBNDS,&
   & POROG,ZOOROG,FPBL,ZTSI,ZTSO,ZSPP,ZPREPP,ZPREH2,&
   & ZTT0,ZUT0,ZVT0,ZDEP0,ZWVEL0,ZGFLT0,&
   & ZI_PRESH,ZI_PRESF,ZI_XYB(:,:,YYTXYB%M_LNPR),ZI_XYB(:,:,YYTXYB%M_ALPH),ZI_R,ZI_KAPA,ZI_RHF,&
   & ZTPP,ZUPP,ZVPP,ZPDEPP,ZWHPP2,ZWFPP,&
   & ZGFLPP(1:KPROMA,1:NFLEVG,1:IPPGFL),ZPHIPP,ZHUPP,ZPEORG)

  IF (LLS1_GFL(YLPPIC%IG)) THEN
    ZGRHLPP(KST:KND,1:NFLEVG)=ZGFLPP(KST:KND,1:NFLEVG,YLPPIC%IG)
  ELSE
    ZGRHLPP(KST:KND,1:NFLEVG)=0._JPRB
  ENDIF
  IF (LLS1_GFL(YLPPIC%IH)) THEN
    ZGRHLPP(KST:KND,1:NFLEVG)=&
     & ZGRHLPP(KST:KND,1:NFLEVG)+ZGFLPP(KST:KND,1:NFLEVG,YLPPIC%IH)
  ENDIF

  SELECT CASE (LL_WWS)
  CASE (.TRUE.)
    DO JI=KST, KND
      ZWSPP(JI)=ZWHPP2(JI,NFLEVG,1)
    ENDDO
  END SELECT

  IF (LLPBL) THEN

    ! We find there all calls to GP, GNH routines for NFLEVG-target system
    ! (using ._PP as input data).

    ! R:
    CALL GPRCP(KPROMA,KST,KND,NFLEVG,&
     & PQ=ZGFLPP(1,1,YLPPIC%IQ),PQI=ZGFLPP(1,1,YLPPIC%II),&
     & PQL=ZGFLPP(1,1,YLPPIC%IL),PQR=ZGFLPP(1,1,YLPPIC%IRR),&
     & PQS=ZGFLPP(1,1,YLPPIC%IS),PQG=ZGRHLPP,PCP=ZCPPP,PR=ZRPP,PKAP=ZKAPPP)

    ! Pressures:
    ZPRES_PPH(KST:KND,NFLEVG)=ZSPP(KST:KND)
    CALL GPHPRE(KPROMA,NFLEVG,KST,KND,YRVAB,ZPRES_PPH,PXYB=ZXYB_PP,PRESF=ZPRES_PPF)

    ! Compute (pre/prehyd)*ZRPP for input to GPGEO.
    DO JL=1,NFLEVG
      DO JI=KST,KND
        ZRRED_PP(JI,JL)=(1.0_JPRB+ZPDEPP(JI,JL)/ZPRES_PPF(JI,JL))*ZRPP(JI,JL)
      ENDDO 
    ENDDO

    ! Geopotential height:
    ZGEO_PPH(KST:KND,NFLEVG)=ZOOROG(KST:KND)
    CALL GPGEO(KPROMA,KST,KND,NFLEVG,&
     & ZGEO_PPH,ZGEO_PPF,ZTPP,ZRRED_PP,ZXYB_PP(:,:,YYTXYB%M_LNPR),ZXYB_PP(:,:,YYTXYB%M_ALPH))

    CALL FPACHMT(KPROMA,KST,KND,NFLEVG,&
    & ZTPP,ZGFLPP(1,1,YLPPIC%IQ),ZUPP,ZVPP,&
    & ZPRES_PPH,ZPRES_PPF,ZCPPP,ZRPP,ZGEO_PPH,ZGEO_PPF,&
    & PCLSM,PTS0,PSR0,PTR0,PCVEG,PCARG,PCD2,PCIVE,&
    & PCSAB,PSD0,PSW0,PSF0,PHV0,ZQCLS,ZRHCLS,ZTCLS,ZUCLS,ZVCLS,ZFCLS,ZUGST, &
    & ZVGST,ZFGST,ZPCLS)  

  ENDIF

  IF (LLICV) THEN

    ! convert model levels to height levels
    ICAPE=NFPCAPE
    IF (LFPCAPEX) THEN
      DO JI=KST,KND
        ZPCLS(JI)=ZSPP(JI)
      ENDDO
      CALL FPCICA(KST,KND,KPROMA,NFLEVG,ICAPE,PTCLS,ZPCLS,PRHCLS,ZTPP, &
       & ZPREPP,ZGFLPP(1,1,YLPPIC%IQ),ZCAPE,ZCIN,ZTCVS,ILCL,IFCL,IEL)  
    ELSE
      CALL FPCICA(KST,KND,KPROMA,NFLEVG,ICAPE,ZTCLS,ZPCLS,ZRHCLS,ZTPP, &
       & ZPREPP,ZGFLPP(1,1,YLPPIC%IQ),ZCAPE,ZCIN,ZTCVS,ILCL,IFCL,IEL)  
    ENDIF

    IF (QFPTYPE%LL(TFP_LCL%ICOD)) THEN
      DO JI=KST,KND
        IF (ILCL(JI) > 0 .AND. ILCL(JI) <= NFLEVG) THEN
          ZLCL(JI)=ZI_GEOPF(JI,ILCL(JI))
        ELSE
          ZLCL(JI)=0._JPRB
        ENDIF
      ENDDO
    ENDIF

    IF (QFPTYPE%LL(TFP_FCL%ICOD)) THEN
      DO JI=KST,KND
        IF (IFCL(JI) > 0 .AND. IFCL(JI) <= NFLEVG) THEN
          ZFCL(JI)=ZI_GEOPF(JI,IFCL(JI))
        ELSE
          ZFCL(JI)=0._JPRB
        ENDIF
      ENDDO
    ENDIF

    IF (QFPTYPE%LL(TFP_EL%ICOD)) THEN
      DO JI=KST,KND
        IF (IEL(JI) > 0 .AND. IEL(JI) <= NFLEVG) THEN
          ZEL(JI)=ZI_GEOPF(JI,IEL(JI))
        ELSE
          ZEL(JI)=0._JPRB
        ENDIF
      ENDDO
    ENDIF

  ENDIF

ENDIF

IF (LLAPA(2)) THEN

  LLWHALF=.FALSE.
  LLGEOP=.TRUE.

  CALL APACHE(KPROMA,KST,KND,NFLEVG,NFLEVG,IPPGFL,NPPM,YLPPIC,&
   & LLNHPD,LLWHALF,LLWFULL,LLS2_GFL,LLGEOP,LLSPCH,ZRHBNDS,&
   & POROG,ZOOROG,FPBL,ZTSI,ZTSA,ZSPP,ZPREPP,ZPREH2,&
   & ZTT0,ZUT0,ZVT0,ZDEP0,ZWVEL0,ZGFLT0,&
   & ZI_PRESH,ZI_PRESF,ZI_XYB(:,:,YYTXYB%M_LNPR),ZI_XYB(:,:,YYTXYB%M_ALPH),ZI_R,ZI_KAPA,ZI_RHF,&
   & ZTPP,ZUPP,ZVPP,ZPDEPP,ZWHPP2,ZWFPP,&
   & ZGFLPP(1:KPROMA,1:NFLEVG,1:IPPGFL),ZPHIPP,ZHUPP,ZPEORG)
  
  IF (LL_HTB .OR. LL_PTB) THEN
    IF (NFP3I > 0) THEN
      DO JLEVP=1,NFP3I
        ZXTEMP=ABS(RFP3I(JLEVP))
        LLTOP = (RFP3I(JLEVP) < 0)
        CALL PPLTEMP(KPROMA,KST,KND,NFLEVG,ZPHIPP,ZTPP,ZXTEMP,LLTOP,ZHTB(1,JLEVP))
      ENDDO
      IF (LL_PTB) THEN
        CALL FPPS(KPROMA,KST,KND,NFLEVG,NFP3I,ZI_GEOPH,ZHTB,ZTT0(1,1), &
         & ZI_R,ZI_PRESH,ZTSI,ZPTB)
      ENDIF
      IF (LL_HTB) THEN
        ! Convert from geopotentials to meters
        DO JLEVP=1,NFP3I
          ZHTB(KST:KND,JLEVP)=ZHTB(KST:KND,JLEVP)*Z1SG
        ENDDO
      ENDIF
    ENDIF
  ENDIF

  IF (LL_HTPW) THEN
    ITOP=1
    DO JL=1,NFLEVG
      IF (STPRE(JL) >= PPROOF) THEN
        ITOP=JL
        EXIT
      ENDIF
    ENDDO
    DO JL=ITOP,NFLEVG
      CALL PPWETPOINT(KST,KND,KPROMA,ZPREPP(1,JL),ZTPP(1,JL),&
       & ZGFLPP(1,JL,YLPPIC%IQ),ZGFLPP(1,JL,YLPPIC%IL),ZGFLPP(1,JL,YLPPIC%II),&
       & ZTPWPP(1,JL))  
    ENDDO
    DO JL=1,ITOP-1
      DO JI=KST,KND
        ZTPWPP(JI,JL)=ZTPWPP(JI,ITOP)
      ENDDO
    ENDDO
    LLTOP=.FALSE.
    ZXTEMP=RTT
    CALL PPLTEMP(KPROMA,KST,KND,NFLEVG,ZPHIPP,ZTPWPP,ZXTEMP,LLTOP,ZHTPW)
    ! Convert from geopotentials to meters
    DO JI=KST,KND
      ZHTPW(JI)=ZHTPW(JI)*Z1SG
    ENDDO
  ENDIF

  IF (LLAERO) THEN
    IPOL2=4
    CALL POAERO(ZPHIPP,ZTPP,ZUPP,ZVPP,ZPICAO,ZTICAO,ZUJET,ZVJET,ZPJET,ZPREPP,&
     & KPROMA,KST,KND,NFLEVG,IPOL2)  
  ENDIF

ENDIF

IF (ALLOCATED(ZWHPP2)) DEALLOCATE(ZWHPP2)
IF (ALLOCATED(ZPREH2)) DEALLOCATE(ZPREH2)

IF (QFPTYPE%LL(TFP_TX%ICOD)) THEN
  DO JI=KST,KND
    ZTMAX(JI)=ZTCLS(JI)+PTXI(JI)
  ENDDO
ENDIF

IF (QFPTYPE%LL(TFP_TN%ICOD)) THEN
  DO JI=KST,KND
    ZTMIN(JI)=ZTCLS(JI)+PTNI(JI)
  ENDDO
ENDIF

IF (QFPTYPE%LL(TFP_HUX%ICOD)) THEN
  DO JI=KST,KND
    ZHUMAX(JI)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZRHCLS(JI)+PHUXI(JI)))
  ENDDO
ENDIF

IF (QFPTYPE%LL(TFP_HUN%ICOD)) THEN
  DO JI=KST,KND
    ZHUMIN(JI)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZRHCLS(JI)+PHUNI(JI)))
  ENDDO
ENDIF

!*    2.2   UPPER AIR POST-PROCESSING

IF (KOPLEV > 0) THEN

  !      2.2.1   Compute pressure on post-processing levels
  !              ------------------------------------------

  ALLOCATE(ZWHPP2(KPROMA,KOPLEV,0:1))
  ALLOCATE(ZPREH2(KPROMA,KOPLEV,0:1))
  SELECT CASE (CDCONF)
  CASE ('E')
    CALL PPLETA(KPROMA,KST,KND,NFPLEV,KOPLEV,PXLEV,ZSPP, &
     & ZPREPP,ZPREH2(1,1,0),ZPREH2(1,1,1),PVAH=FPVALH,PVBH=FPVBH)  
  CASE ('Z')
    DO JL=1,KOPLEV
      DO JI=KST,KND
        ZGEOP(JI,JL)=RG*PXLEV(JL)+ZOOROG(JI)
      ENDDO
    ENDDO
    CALL FPPS(KPROMA,KST,KND,NFLEVG,KOPLEV,ZI_GEOPH,ZGEOP,ZTT0(1,1), &
     & ZI_R,ZI_PRESH,ZTSI,ZPREPP)
  CASE ('L')
    DO JL=1,KOPLEV
      DO JI=KST,KND
        ZGEOP(JI,JL)=RG*PXLEV(JL)
      ENDDO
    ENDDO
    CALL FPPS(KPROMA,KST,KND,NFLEVG,KOPLEV,ZI_GEOPH,ZGEOP,ZTT0(1,1), &
     & ZI_R,ZI_PRESH,ZTSI,ZPREPP)  
  CASE DEFAULT 
    CALL ABOR1('ENDPOS : INTERNAL ERROR CDCONF')
  END SELECT

  !      2.2.2   Vertical interpolations on basic fields
  !              ---------------------------------------

  LLGEOP =LL_Z
  LLWHALF=LL_VD
  LLWFULL=LL_VW
  LLNHPD =LL_PD.OR.LL_RHO
  LLSPCH=LFPQ

  CALL APACHE(KPROMA,KST,KND,NFLEVG,KOPLEV,IPPGFL,NPPM,YLPPIC,&
   & LLNHPD,LLWHALF,LLWFULL,LL_GFL,LLGEOP,LLSPCH,ZRHBNDS,&
   & POROG,ZOOROG,FPBL,ZTSI,ZTSO,ZSPP,ZPREPP,ZPREH2,&
   & ZTT0,ZUT0,ZVT0,ZDEP0,ZWVEL0,ZGFLT0,&
   & ZI_PRESH,ZI_PRESF,ZI_XYB(:,:,YYTXYB%M_LNPR),ZI_XYB(:,:,YYTXYB%M_ALPH),ZI_R,ZI_KAPA,ZI_RHF,&
   & ZTPP,ZUPP,ZVPP,ZPDEPP,ZWHPP2,ZWFPP,&
   & ZGFLPP(1:KPROMA,1:KOPLEV,1:IPPGFL),ZPHIPP,ZHUPP,ZPEORG)  

  IF (LL_GFL(YLPPIC%IG)) THEN
    ZGRHLPP(KST:KND,1:KOPLEV)=ZGFLPP(KST:KND,1:KOPLEV,YLPPIC%IG)
  ELSE
    ZGRHLPP(KST:KND,1:KOPLEV)=0._JPRB
  ENDIF
  IF (LL_GFL(YLPPIC%IH)) THEN
    ZGRHLPP(KST:KND,1:KOPLEV)=&
     & ZGRHLPP(KST:KND,1:KOPLEV)+ZGFLPP(KST:KND,1:KOPLEV,YLPPIC%IH)
  ENDIF

  DO JL=1,KOPLEV
    DO JI=KST,KND
      ZHUPP(JI,JL)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZHUPP(JI,JL)))
    ENDDO
  ENDDO

  !      2.2.3   Computation of elaborated fields from basic ones
  !              ------------------------------------------------

  !      2.2.3.1 Vertical divergence quantity -G*(Delta w).

  IF (LL_VD) THEN
    DO JL=1,KOPLEV
      DO JI=KST,KND
        ZMGDWPP(JI,JL)=RG*(ZWHPP2(JI,JL,1)-ZWHPP2(JI,JL,0))
      ENDDO
    ENDDO
  ENDIF

  !      2.2.3.2 Potential and equivalent potential temperatures
  !              and isobaric equivalent temperature.

  IF (LL_TH .OR. LL_ETH .OR. LL_IET) THEN
    ! We find there all calls to GP, GNH routines for KOPLEV-target system
    ! (using ._PP as input data).
    CALL GPRCP(KPROMA,KST,KND,KOPLEV,PQ=ZGFLPP(1,1,YLPPIC%IQ),&
     & PQI=ZGFLPP(1,1,YLPPIC%II),PQL=ZGFLPP(1,1,YLPPIC%IL),&
     & PQR=ZGFLPP(1,1,YLPPIC%IRR),&
     & PQS=ZGFLPP(1,1,YLPPIC%IS),PQG=ZGRHLPP,PCP=ZCPPP,PR=ZRPP,PKAP=ZKAPPP)
    IF (LL_TH .OR. LL_ETH) THEN
      CALL GPTET(KPROMA,KST,KND,KOPLEV,ZPREPP,ZTPP,ZKAPPP,ZTETAPP)
      IF (LL_ETH) THEN
        CALL GPEPT(KPROMA,KST,KND,KOPLEV,ZTETAPP,ZTPP,ZPREPP,ZEPTHPP)
      ENDIF
    ENDIF
    IF (LL_IET) THEN
      CALL GPIET(KPROMA,KST,KND,KOPLEV,&
       & ZGFLPP(1,1,YLPPIC%IQ),ZTPP,ZCPPP,ZPREPP,ZIBETPP)
    ENDIF
  ENDIF

  !      2.2.3.3 Relative moisture in %

  IF (LL_HU .AND. LFPRH100) THEN
    DO JL=1,KOPLEV
      DO JI=KST,KND
        ZHUPP(JI,JL)=ZHUPP(JI,JL)*100.0_JPRB
      ENDDO
    ENDDO
  ENDIF

  !      2.2.3.4 Wet bulb potential temperature

  IF (LL_THPW) THEN
    DO JL=1,KOPLEV
      CALL PPTHPW(KST,KND,KPROMA,ZPREPP(1,JL),ZTPP(1,JL),ZGFLPP(1,JL,YLPPIC%IQ), &
       & ZGFLPP(1,JL,YLPPIC%IL),ZGFLPP(1,JL,YLPPIC%II),ZTHPWPP(1,JL))  
    ENDDO
  ENDIF

  !      2.2.3.5 Wind velocity

  IF (LL_WND) THEN
    DO JL=1,KOPLEV
      DO JI=KST,KND
        ZWINDPP(JI,JL)=SQRT(ZUPP(JI,JL)*ZUPP(JI,JL)+ZVPP(JI,JL)*ZVPP(JI,JL))
      ENDDO
    ENDDO
  ENDIF

  !     2.2.3.6. Simulated Reflectivities

  IF (LL_SRE) THEN
    LLUNIT=.TRUE.
    CALL GPPRS0D(KPROMA,KST,KND,KOPLEV,ZTPP,ZGFLPP(1,1,YLPPIC%IQ),&
     & ZGFLPP(1,1,YLPPIC%IL),ZGFLPP(1,1,YLPPIC%IRR), &
     & ZGFLPP(1,1,YLPPIC%II),ZGFLPP(1,1,YLPPIC%IS),ZGRHLPP,LLUNIT,ZSIMRPP)
  ENDIF

  !     2.2.3.7. Virtual theat

  IF (LL_THV) THEN
    CALL PPCVIRT(KPROMA,KST,KND,KOPLEV,ZTETAPP,ZGFLPP(1,1,YLPPIC%IQ),&
     & ZGFLPP(1,1,YLPPIC%IL),ZGFLPP(1,1,YLPPIC%IRR), &
     & ZGFLPP(1,1,YLPPIC%IS),ZGRHLPP,ZGFLPP(1,1,YLPPIC%II),ZVIRTPP)
  ENDIF
  
  !     2.2.3.8. Humid density

  IF (LL_RHO) THEN
    ! R as function of Qi, QL, Qv only
    CALL GPRCP(KPROMA,KST,KND,KOPLEV,&
     & PQ=ZGFLPP(1,1,YLPPIC%IQ),PQI=ZGFLPP(1,1,YLPPIC%II),&
     & PQL=ZGFLPP(1,1,YLPPIC%IL),PR=ZR2PP)
    DO JL=1,KOPLEV
      DO JI= KST,KND
      ZRHOPP(JI,JL)=(ZPREPP(JI,JL) + ZPDEPP(JI,JL))/&
     & (ZR2PP(JI,JL)*ZTPP(JI,JL))
      ENDDO
    ENDDO
  ENDIF

  DEALLOCATE(ZWHPP2)
  DEALLOCATE(ZPREH2)

ENDIF ! (KOPLEV > 0)

!     ------------------------------------------------------------------

!*       3.     STORE POST-PROCESSED FIELDS.
!               ----------------------------

!*    3.1   GFL FIELDS

DO JV=1,IPPGFL
  ZPPIN(1:KPROMA,1:IMAXLEV)=ZGFLPP(1:KPROMA,1:IMAXLEV,JV)
  CALL STORE_DATA(IGFLCOD(JV),PDATA2=ZPPIN)
ENDDO

!*    3.2   OTHER FIELDS

! Store data according to the derivative order and the shape of array:

CALL STORE_DATA(TFP_SP%ICOD  ,PDATA1=ZSPP   )
CALL STORE_DATA(TFP_LNSP%ICOD,PDATA1=ZLNSP  )
CALL STORE_DATA(TFP_WWS%ICOD ,PDATA1=ZWSPP  )
CALL STORE_DATA(TFP_UCLS%ICOD,PDATA1=ZUCLS  )
CALL STORE_DATA(TFP_VCLS%ICOD,PDATA1=ZVCLS  )
CALL STORE_DATA(TFP_TCLS%ICOD,PDATA1=ZTCLS  )
CALL STORE_DATA(TFP_QCLS%ICOD,PDATA1=ZQCLS  )
CALL STORE_DATA(TFP_RCLS%ICOD,PDATA1=ZRHCLS )
CALL STORE_DATA(TFP_FCLS%ICOD,PDATA1=ZFCLS  )
CALL STORE_DATA(TFP_TN%ICOD  ,PDATA1=ZTMIN  )
CALL STORE_DATA(TFP_TX%ICOD  ,PDATA1=ZTMAX  )
CALL STORE_DATA(TFP_HUX%ICOD ,PDATA1=ZHUMAX )
CALL STORE_DATA(TFP_HUN%ICOD ,PDATA1=ZHUMIN )
CALL STORE_DATA(TFP_UGST%ICOD,PDATA1=ZUGST  )
CALL STORE_DATA(TFP_VGST%ICOD,PDATA1=ZVGST  )
CALL STORE_DATA(TFP_FGST%ICOD,PDATA1=ZFGST  )
CALL STORE_DATA(TFP_CAPE%ICOD,PDATA1=ZCAPE  )
CALL STORE_DATA(TFP_CIEN%ICOD,PDATA1=ZCIN   )
CALL STORE_DATA(TFP_TCVS%ICOD,PDATA1=ZTCVS  )
CALL STORE_DATA(TFP_HTPW%ICOD,PDATA1=ZHTPW  )
CALL STORE_DATA(TFP_UJET%ICOD,PDATA1=ZUJET  )
CALL STORE_DATA(TFP_VJET%ICOD,PDATA1=ZVJET  )
CALL STORE_DATA(TFP_PJET%ICOD,PDATA1=ZPJET  )
CALL STORE_DATA(TFP_PCAO%ICOD,PDATA1=ZPICAO )
CALL STORE_DATA(TFP_TCAO%ICOD,PDATA1=ZTICAO )

CALL STORE_DATA(TFP_PTB%ICOD ,PDATA2=ZPTB  ) 
CALL STORE_DATA(TFP_HTB%ICOD ,PDATA2=ZHTB  )
CALL STORE_DATA(TFP_U%ICOD   ,PDATA2=ZUPP   )
CALL STORE_DATA(TFP_V%ICOD   ,PDATA2=ZVPP   )
CALL STORE_DATA(TFP_T%ICOD   ,PDATA2=ZTPP   )
CALL STORE_DATA(TFP_Z%ICOD   ,PDATA2=ZPHIPP )
CALL STORE_DATA(TFP_HU%ICOD  ,PDATA2=ZHUPP  )
CALL STORE_DATA(TFP_TH%ICOD  ,PDATA2=ZTETAPP)
CALL STORE_DATA(TFP_THPW%ICOD,PDATA2=ZTHPWPP)
CALL STORE_DATA(TFP_ETH%ICOD ,PDATA2=ZEPTHPP)
CALL STORE_DATA(TFP_IET%ICOD ,PDATA2=ZIBETPP)
CALL STORE_DATA(TFP_WND%ICOD ,PDATA2=ZWINDPP)
CALL STORE_DATA(TFP_P%ICOD   ,PDATA2=ZPREPP )
CALL STORE_DATA(TFP_PD%ICOD  ,PDATA2=ZPDEPP )
CALL STORE_DATA(TFP_VD%ICOD  ,PDATA2=ZMGDWPP)
CALL STORE_DATA(TFP_VW%ICOD  ,PDATA2=ZWFPP  )
CALL STORE_DATA(TFP_SRE%ICOD ,PDATA2=ZSIMRPP)
CALL STORE_DATA(TFP_THV%ICOD ,PDATA2=ZVIRTPP)
CALL STORE_DATA(TFP_RHO%ICOD ,PDATA2=ZRHOPP)
CALL STORE_DATA(TFP_LCL%ICOD ,PDATA1=ZLCL)
CALL STORE_DATA(TFP_FCL%ICOD ,PDATA1=ZFCL)
CALL STORE_DATA(TFP_EL%ICOD ,PDATA1=ZEL)

DO JV=1,JPOSVX2
  I=TFP_FUA(JV)%ICOD
  IF (QFPTYPE%LL(I)) THEN
    DO JL=1,QFPTYPE%ILEV(I)
      ILOC=QFPTYPE%IGFP(I)+JL-1
      DO JI=KST,KND 
        PFP(JI,ILOC)=0.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

!*       4.     DEALLOCATIONS.
!               --------------

IF (ALLOCATED(ZGFLPP)) DEALLOCATE(ZGFLPP)
IF (ALLOCATED(IGFLCOD)) DEALLOCATE(IGFLCOD)
IF (ALLOCATED(IDIN_GFL)) DEALLOCATE(IDIN_GFL)
IF (ALLOCATED(LL_GFL)) DEALLOCATE(LL_GFL)
IF (ALLOCATED(LLS1_GFL)) DEALLOCATE(LLS1_GFL)
IF (ALLOCATED(LLS2_GFL)) DEALLOCATE(LLS2_GFL)
IF (ALLOCATED(LLIN_GFL)) DEALLOCATE(LLIN_GFL)
IF (ALLOCATED(ZGFLT0)) DEALLOCATE(ZGFLT0)

!------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ENDPOS',1,ZHOOK_HANDLE)
!------------------------------------------------------------------
CONTAINS

SUBROUTINE STORE_DATA(K,PDATA1,PDATA2)

!------------------------------------------------------------------
INTEGER(KIND=JPIM), INTENT(IN) :: K
REAL(KIND=JPRB)   , INTENT(IN), OPTIONAL :: PDATA1(:)
REAL(KIND=JPRB)   , INTENT(IN), OPTIONAL :: PDATA2(:,:)
!------------------------------------------------------------------
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ENDPOS:STORE_DATA',0,ZHOOK_HANDLE)
!------------------------------------------------------------------

IF (QFPTYPE%LL(K)) THEN
  IF (PRESENT(PDATA1)) THEN
    ILOC=QFPTYPE%IGFP(K)
    SELECT CASE (ABS(TFP_DYNDS(K)%IORDR))
    CASE (0)
      DO JI=KST, KND
        PFP(JI,ILOC) = PDATA1(JI)
      ENDDO
    CASE (1)
      DO JI=KST, KND
        PFP(JI,ILOC) = PDATA1(JI)*Z1SGM(JI)
      ENDDO
    CASE (2)
      DO JI=KST, KND
        PFP(JI,ILOC) = PDATA1(JI)*(Z1SGM(JI)*Z1SGM(JI))
      ENDDO
    END SELECT
  ELSEIF (PRESENT(PDATA2)) THEN
    DO JL=1,QFPTYPE%ILEV(K)
      ILOC=QFPTYPE%IGFP(K)+JL-1
      ILPT=QFPTYPE%ILVP(JL,K)
      SELECT CASE (ABS(TFP_DYNDS(K)%IORDR))
      CASE (0)
        DO JI=KST, KND
          PFP(JI,ILOC) = PDATA2(JI,ILPT)
        ENDDO
      CASE (1)
        DO JI=KST, KND
          PFP(JI,ILOC) = PDATA2(JI,ILPT)*Z1SGM(JI)
        ENDDO
      CASE (2)
        DO JI=KST, KND
          PFP(JI,ILOC) = PDATA2(JI,ILPT)*(Z1SGM(JI)*Z1SGM(JI))
        ENDDO
      END SELECT
    ENDDO
  ELSE
    CALL ABOR1('ENPOS.STORE_DATA : INTERFACE ERROR')
  ENDIF 
ENDIF    
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ENDPOS:STORE_DATA',1,ZHOOK_HANDLE)
END SUBROUTINE STORE_DATA

!     ------------------------------------------------------------------

SUBROUTINE SUPTRPPGFL_ENDPOS(KFL,KPPGFL,KMPP)
! computes pointers to store interpolated GFL variables.

INTEGER(KIND=JPIM), INTENT(IN)     :: KFL ! number of fields
INTEGER(KIND=JPIM), INTENT(INOUT)  :: KPPGFL ! cumulated number of fields
INTEGER(KIND=JPIM), INTENT(OUT)    :: KMPP ! pointer

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('ENDPOS:SUPTRPPGFL_ENDPOS',0,ZHOOK_HANDLE)

KMPP=KPPGFL+1
KPPGFL=KPPGFL+KFL

IF (LHOOK) CALL DR_HOOK('ENDPOS:SUPTRPPGFL_ENDPOS',1,ZHOOK_HANDLE)

END SUBROUTINE SUPTRPPGFL_ENDPOS

!     ------------------------------------------------------------------
END SUBROUTINE ENDPOS
